import random
from threading import Lock
from typing import Dict, List

import nickgen.macros_handler
from nickgen.syllable_eng import SyllableEng
from nickgen.syllable_jap import SyllableJap
from nickgen.syllable_loader import SyllableLoader


class LoginMacrossHandler:
    syllableEng_0: SyllableEng = SyllableLoader.LoadEng()
    syllableEng_1: SyllableEng = SyllableLoader.LoadLat()
    syllableJap_0: SyllableJap = SyllableLoader.LoadJap()
    syllableEng_2: SyllableEng

    macrossHandler_0 = nickgen.macros_handler.MacrossHandler("[", "]", None)
    locker = Lock()

    @classmethod
    def SetUserBase(cls, userBase: SyllableEng):
        LoginMacrossHandler.syllableEng_2 = userBase

    @classmethod
    def GetLogins(cls, macross: str, count: int) -> tuple[str]:
        if count > 0:
            with cls.locker:
                generator = (
                    cls.macrossHandler_0.ExecComplexMacross(macross)
                    for i in range(count)
                )
                return tuple(generator)
        else:
            return None

    @classmethod
    def ExecSimplMacross(cls, simplMacross: str):
        if not simplMacross:
            return ""

        strArray = simplMacross.split("|")
        str1 = strArray[0]

        try:
            if str1 == "Eng":
                return LoginMacrossHandler.syllableEng_0.GetLogins(int(strArray[1]), 1)[
                    0
                ]
            if str1 == "RndNum":
                return str(random.randint(int(strArray[1]), int(strArray[2]) - 1))
            if str1 == "RndText":
                return LoginMacrossHandler.__RndText(strArray)
            if str1 == "Jap":
                return LoginMacrossHandler.syllableJap_0.GetLogins(int(strArray[1]), 1)[
                    0
                ]
            if str1 == "Lat":
                return LoginMacrossHandler.syllableEng_1.GetLogins(int(strArray[1]), 1)[
                    0
                ]
            if str1 == "RndSym":
                return LoginMacrossHandler.__RndSym(strArray)
            if str1 == "V":
                return LoginMacrossHandler.syllableEng_0.GetChar(
                    SyllableEng.CharType.Vowel
                )
            if str1 == "C":
                return LoginMacrossHandler.syllableEng_0.GetChar(
                    SyllableEng.CharType.Consonant
                )
            if str1 == "UserDef":
                return LoginMacrossHandler.syllableEng_2.GetLogins(int(strArray[1]), 1)[
                    0
                ]

            raise Exception("Error in LoginMacrossHandler")
        except:
            pass
        return ""

    @staticmethod
    def __RndText(strArray: List[str]):
        str2 = "0123456789"
        str3 = "QWERTYUIOPASDFGHJKLZXCVBNM"
        str4 = "qwertyuiopasdfghjklzxcvbnm"
        int32_7 = int(strArray[1])
        chArray1: List[str] = []
        flag1 = False
        flag2 = False

        if len(strArray) == 3:
            if "d" in strArray[2] or "D" in strArray[2]:
                flag1 = True
            if "c" in strArray[2] or "C" in strArray[2]:
                flag2 = True

        if flag1:
            str4 += str2
        if flag2:
            str4 += str3

        length1 = len(str4)
        for i in range(int32_7):
            chArray1[i] = str4[random.randint(0, length1 - 1)]
        return "".join(chArray1)

    @staticmethod
    def __RndSym(strArray: List[str]):
        int32_8 = int(strArray[1])
        str5 = strArray[2]
        chArray2: List[str] = []
        length2 = len(str5)
        for i in range(int32_8):
            chArray2[i] = str5[random.randint(0, length2 - 1)]
        return "".join(chArray2)



from importlib.abc import PathEntryFinder
import re
import random
from typing import Pattern, Dict, List

from random import Random
from enum import Enum
from collections import OrderedDict

from nickgen.syllable_chain import SyllableChain
from nickgen.ForSyllableGroups import ForSyllableGroups
from nickgen.syllable_groups import SyllableGroups

class SyllableEng:

    CharType = Enum('CharType', 'Vowel Consonant')

    IsReady :bool #get; set;

    def __init__(self):
        self.string_0 :str = "VoidSyllable"
        self.string_1 :str = "VoidGroup"
        self.DwvbpfyYn :str = "ccgc"
        self.string_2 :str = "cgcc"
        self.string_3 :str = "cgc"
        self.string_4 :str = "cgg"
        self.string_5 :str = "cg"
        self.string_6 :str = "gc"

        self.dictionary_0 :Dict[str, SyllableChain] = dict()
        self.syllableGroups_0 :SyllableGroups = SyllableGroups()
        self.sortedList_0 = list()

        #self.random_0 = random
        self.regex_0 :Pattern = re.compile(r'[bcdfjghklmnpqrstvwxz01234][bcdfjghklmnpqrstvwxz01234][aeiouy56789][bcdfjghklmnpqrstvwxz01234]')
        self.regex_1 :Pattern = re.compile(r'[bcdfjghklmnpqrstvwxz01234][aeiouy56789][bcdfjghklmnpqrstvwxz01234][bcdfjghklmnpqrstvwxz01234](?=[bcdfjghklmnpqrstvwxz01234\\#])')
        self.regex_2 :Pattern = re.compile(r'[bcdfjghklmnpqrstvwxz01234][aeiouy56789][bcdfjghklmnpqrstvwxz01234](?=[bcdfjghklmnpqrstvwxz01234\\#])')
        self.regex_3 :Pattern = re.compile(r'[bcdfjghklmnpqrstvwxz01234][aeiouy56789][aeiouy56789]')
        self.regex_4 :Pattern = re.compile(r'[bcdfjghklmnpqrstvwxz01234][aeiouy56789](?=[bcdfjghklmnpqrstvwxz01234\\#])')
        self.regex_5 :Pattern = re.compile(r'[aeiouy56789][bcdfjghklmnpqrstvwxz01234]')
        self.regex_6 :Pattern = re.compile(r'[a-zA-Z]+')

    def AddTextEn(self, text:str):
        text = text.lower()
        matchCollection = self.regex_6.findall(text)
        if len(matchCollection) == 0:
            return
        
        for capture in matchCollection:
            self.AddWordEn(capture)
        
    def method_1(self, char_0: str):
        #refactor this to constant dict
        table :Dict[str,bool] = {
            '0':False,
            '1':False,
            '2':False,
            '3':False,
            '4':False,
            '5':True,
            '6':True,
            '7':True,
            '8':True,
            '9':True,
            'a':True,
            'b':False,
            'c':False,
            'd':False,
            'e':True,
            'f':False,
            'g':False,
            'h':False,
            'i':True,
            'j':False,
            'k':False,
            'l':True,
            'm':False,
            'n':False,
            'o':True,
            'p':False,
            'q':False,
            'r':False,
            's':False,
            't':False,
            'u':True,
            'v':False,
            'w':False,
            'x':False,
            'y':True,
            'z':False,
        }

        return table[char_0]

    def GetChar(self, charType: CharType):
        # Vowel, Consonant => Гласный, Согласный
        str__ = ''

        if charType is not charType.Vowel:
            str__ = 'bcdfghjklmnpqrstvwxz'
        else:
            str__ = 'aeouyi'

        length = len(str__)
        randValue = random.randint(0, length - 1)
        return str__[randValue]
    
    def AddWordEn(self, word: str):
        # if word in self.sortedList_0:
        #     return

        for tup in self.sortedList_0:
            if word == tup[0]:
                return
        
        self.sortedList_0.append((word,False))
        self.sortedList_0 = sorted(self.sortedList_0, key=lambda x: x[0])
        stringList1 = []
        stringList2 = []
        flag :bool = False

        word = self.method_3(word)
        word += '#'
        if not word:
            return
        
        while (flag is False and word):
            match1 = self.regex_0.search(word)
            if (match1 and match1.pos == 0):
                mValue1 = match1.group(0)
                stringList1.append(mValue1)
                stringList2.append('ccgc')
                word = word[len(mValue1):(len(word) - len(mValue1))]
            else:
                match2 = self.regex_1.search(word)
                if (match2 and match2.pos == 0):
                    mValue2 = match2.group(0)
                    stringList1.append(mValue2)
                    stringList2.append('cgcc')
                    word = word[len(mValue2):(len(word) - len(mValue2))]
                else:
                    match3 = self.regex_2.search(word)
                    if (match3 and match3.pos == 0):
                        mValue3 = match3.group(0)
                        stringList1.append(mValue3)
                        stringList2.append('cgc')
                        word = word[len(mValue3):(len(word) - len(mValue3))]
                    else:
                        match4 = self.regex_3.search(word)
                        if (match4 and match4.pos == 0):
                            mValue4 = match4.group(0)
                            stringList1.append(mValue4)
                            stringList2.append('cgg')
                            word = word[len(mValue4):(len(word) - len(mValue4))]
                        else:
                            match5 = self.regex_4.search(word)
                            if (match5 and match5.pos == 0):
                                mValue5 = match5.group(0)
                                stringList1.append(mValue5)
                                stringList2.append('cg')
                                word = word[len(mValue5):(len(word) - len(mValue5))]
                            else:
                                match6 = self.regex_5.search(word)
                                if (match6 and match6.pos == 0):
                                    mValue6 = match6.group(0)
                                    stringList1.append(mValue6)
                                    stringList2.append('gc')
                                    word = word[len(mValue6):(len(word) - len(mValue6))]
                                else:
                                    flag = True

        for i, item in enumerate(stringList1):
            stringList1[i] = self.method_4(item)

        stringList1.append('VoidSyllable')
        stringList1.append('VoidGroup')

        for i, item in enumerate(stringList1[:-1]):
            if item in self.dictionary_0:
                self.dictionary_0[item].AddForwardSyllable(stringList1[i + 1])
            else:
                syllableChain :SyllableChain = SyllableChain(item)
                self.dictionary_0[syllableChain.Name] = syllableChain
                syllableChain.AddForwardSyllable(stringList1[i + 1])

        for i, item in enumerate(stringList2[:-1]):
            self.syllableGroups_0.AddSyllableInGroup(item, stringList1[i])
            self.syllableGroups_0.AddForwardGroup(item, stringList2[i + 1])

        pass

    def method_2(self, string_7: str):
        matches = self.regex_6.findall(string_7)
        return ''.join(matches)
    
    def method_3(self, string_7: str):
        string_7 = string_7.replace('ch', '0')
        string_7 = string_7.replace('ph', '1')
        string_7 = string_7.replace('th', '2')
        string_7 = string_7.replace('zh', '3')
        string_7 = string_7.replace('wh', '4')
        string_7 = string_7.replace('sh', '@')
        string_7 = string_7.replace('ee', '5')
        string_7 = string_7.replace('ea', '6')
        string_7 = string_7.replace('oo', '7')
        string_7 = string_7.replace('ua', '8')
        string_7 = string_7.replace('ou', '9')
        return string_7

    def method_4(self, string_7: str):
        string_7 = string_7.replace("0", "ch")
        string_7 = string_7.replace("1", "ph")
        string_7 = string_7.replace("2", "th")
        string_7 = string_7.replace("3", "zh")
        string_7 = string_7.replace("4", "wh")
        string_7 = string_7.replace("@", "sh")
        string_7 = string_7.replace("5", "ee")
        string_7 = string_7.replace("6", "ea")
        string_7 = string_7.replace("7", "oo")
        string_7 = string_7.replace("8", "ua")
        string_7 = string_7.replace("9", "ou")
        return string_7

    def GetMatrixSyllable(self):
        #no references
        pass

    def GetMatrixGroup(self):
        #no references
        return self.syllableGroups_0.GetMatrix()

    def GetLogins(self, _length:int, count:int):
        return self.syllableGroups_0.GetLogins(_length, count)

    def Save(self, path: str):
        #no references
        pass

    @staticmethod
    def Load(path: str):
        try:
            import json
            json_dict = {}
            with open(path, 'r') as file:
                json_dict = json.load(file)
            
            forSyllableGroups :ForSyllableGroups = ForSyllableGroups.from_json(json_dict)
            # forSyllableGroups :ForSyllableGroups = ForSyllableGroups(**json_dict)

            sEng = SyllableEng()
            sEng.syllableGroups_0 = SyllableGroups()
            sEng.syllableGroups_0.fsg = forSyllableGroups
            return sEng
        except:
            raise

    def GoogleTest(self, letters: str, countUse: str):
        #no references
        pass
        return self.syllableGroups_0.GoogleTest(letters, countUse)


import random
import re
from typing import Dict, List

from nickgen.syllable_chain import SyllableChain


class SyllableJap:

    def __init__(self):
        #constants
        self.string_0 = 'VoidSyllable'
        self.string_1 = 'VoidGroup'

        self.string_2 = ['ka', 'ki', 'ku', 'ke', 'ko', 'kya', 'kyu', 'kyo', 'sa', 'shi', 'su', 'se', 'so', 'sha', 'shu', 'sho', 'ta', 'chi', 'tsu', 'te', 'to', 'cha', 'chu', 'cho', 'na', 'ni', 'nu', 'ne', 'no', 'nya', 'nyu', 'nyo', 'ha', 'hi', 'fu', 'he', 'ho', 'hya', 'hyu', 'hyo', 'ma', 'mi', 'mu', 'me', 'mo', 'mya', 'myu', 'myo', 'ya', 'yu', 'yo', 'ra', 'ri', 'ru', 're', 'ro', 'rya', 'ryu', 'ryo', 'wa', 'wi', 'we', 'wo', 'ga', 'gi', 'gu', 'ge', 'go', 'gya', 'gyu', 'gyo', 'za', 'ji', 'zu', 'ze', 'zo', 'ja', 'ju', 'jo', 'da', 'ji', 'dzu', 'de', 'do', 'ja', 'ju', 'jo', 'ba', 'bi', 'bu', 'be', 'bo', 'bya', 'byu', 'byo', 'pa', 'pi', 'pu', 'pe', 'po', 'pya', 'pyu', 'pyo', 'a', 'i', 'u', 'e', 'o', 'n', 'vu']
        self.string_3 = ['か', 'き', 'く', 'け', 'こ', 'きゃ', 'きゅ', 'きょ', 'さ', 'し', 'す', 'せ', 'そ', 'しゃ', 'しゅ', 'しょ', 'た', 'ち', 'つ', 'て', 'と', 'ちゃ', 'ちゅ', 'ちょ', 'な', 'に', 'ぬ', 'ね', 'の', 'にゃ', 'にゅ', 'にょ', 'は', 'ひ', 'ふ', 'へ', 'ほ', 'ひゃ', 'ひゅ', 'ひょ', 'ま', 'み', 'む', 'め', 'も', 'みゃ', 'みゅ', 'みょ', 'や', 'ゆ', 'よ', 'ら', 'り', 'る', 'れ', 'ろ', 'りゃ', 'りゅ', 'りょ', 'わ', 'ゐ', 'ゑ', 'を', 'が', 'ぎ', 'ぐ', 'げ', 'ご', 'ぎゃ', 'ぎゅ', 'ぎょ', 'ざ', 'じ', 'ず', 'ぜ', 'ぞ', 'じゃ', 'じゅ', 'じょ', 'だ', 'ぢ', 'づ', 'で', 'ど', 'ぢゃ', 'ぢゅ', 'ぢょ', 'ば', 'び', 'ぶ', 'べ', 'ぼ', 'びゃ', 'びゅ', 'びょ', 'ぱ', 'ぴ', 'ぷ', 'ぺ', 'ぽ', 'ぴゃ', 'ぴゅ', 'ぴょ', 'あ', 'い', 'う', 'え', 'お', 'ん', 'ゔ']
        self.string_4 = '(?<=\\s)[かきくけこきゃきゅきょさしすせそしゃしゅしょたちつてとちゃちゅちょなにぬねのにゃにゅにょはひふへほひゃひゅひょまみむめもみゃみゅみょやゆよらりるれろりゃりゅりょわゐゑをがぎぐげごぎゃぎゅぎょざじずぜぞじゃじゅじょだぢづでどぢゃぢゅぢょばびぶべぼびゃびゅびょぱぴぷぺぽぴゃぴゅぴょあいうえおんゔ]+'
        self.sortedList_0 = []
        self.dictionary_0 :Dict[str,SyllableChain] = dict()
        self.arrayList_0 = None

    def GetJapWord(self, countOfBlocks:int):
        empty = ''
        for _ in range(countOfBlocks):
            empty += self.string_2[random.randint(0, len(self.string_2) - 1)]
        return empty

    def AddText(self, text:str):
        matches = re.findall(self.string_4, text)
        if len(matches) <= 0:
            return
        for capture in matches:
            self.method_1(capture)

    def method_1(self, string_5 :str):
        if not string_5:
            return
        
        stringList = []
        flag1 = False
        while not flag1 and string_5:
            flag2 = False
            for index in range(len(self.string_3)):
                if string_5.startswith(self.string_3[index]):
                    if self.string_3[index] != 'う':
                        stringList.append(self.string_2[index])
                    string_5 = string_5[0:len(self.string_3[index])]
                    flag2 = True
                    break
            if not flag2:
                flag1 = True
        
        key = ''.join(stringList)
        for tup in self.sortedList_0:
            if key == tup[0]:
                return

        # что-то неправильно?
        if key in self.sortedList_0:
            return
        self.sortedList_0.append((key, False))
        self.sortedList_0 = sorted(self.sortedList_0, key=lambda x: x[0])
        stringList.append('VoidSyllable')

        for i, item in enumerate(stringList[:-1]):
            if item in self.dictionary_0:
                self.dictionary_0[item].AddForwardSyllable(stringList[i + 1])
            else:
                #syllableChain = SyllableChain(item)
                pass

    def GetMatrixSyllable(self):
        stringList :List[str]= List()
        numArray = [ [],[] ]

        for key in self.dictionary_0.keys():
            stringList.append(key)
        
        for i, item in enumerate(stringList):
            numArray[i] = []

            for i2, item2 in enumerate(stringList):
                numArray[i][i2] = self.dictionary_0[item].GetCount(item2)

        arrayList = []
        arrayList.append(stringList)
        arrayList.append(numArray)
        self.arrayList_0 = arrayList
        return arrayList

    def GetLogins(self, lenght :int, count : int):
        if self.arrayList_0 is None:
            self.arrayList_0 = self.GetMatrixSyllable()
        
        stringList :List[str] = []
        strArray = list(self.arrayList_0[0])
        numArray = self.arrayList_0[1]

        for _ in range(count):
            str1 = ''
            index2 = random.randint(0, len(strArray) - 1)
            str2 = str1 + strArray[index2]
            for _ in range(lenght -1):
                maxValue = 0
                num1 = 0
                for index4 in range(len(numArray) -1):
                    maxValue += numArray[index2][index4]
                num2 = random.randint(0, maxValue - 1)
                num3 = 0
                for index4 in range(len(numArray) -1):
                    num3 += numArray[index2][index4]
                    if num1 > num2 or num2 >= num3:
                        num1 = num3
                    else:
                        index2 = index4
                        break
                str2 += strArray[index2]
            stringList.append(str2)
        return stringList

    def Save(self, path :str):
        #no references
        pass

    @staticmethod
    def Load(path :str):
        try:
            import json
            json_dict = {}
            with open(path, 'r') as file:
                json_dict = json.load(file)
            
            # arrayList = SyllableJap.arrayList_from_json(json_dict)
            arrayList = json_dict
            sJap = SyllableJap()
            sJap.arrayList_0 = arrayList
            return sJap
        except:
            #костыль!
            raise
            #raise Exception('Cant load SyllableJap')
        

    @classmethod
    def arrayList_from_json(cls, data):
        # import json
        # fp = open('test.json', 'w')
        # json.dump(data, fp, indent=4)
        # fp.close()
        try:
            result = []

            
            return result
        except:
            raise
            #raise Exception('Error loading arrayList_from_json()')


from typing import Pattern, Match
import re
import nickgen.login_macros_handler

class MacrossHandler:

    def __init__(self, openBracket: str, closeBracket: str, execSimlpeMacrossDeligate):
        self.string_0 = 'Difficult macros processing'
        self.string_1 = 'Simple macros processing feature is not assigned'
        self.string_2 = 'Macros parsing'
        self.string_3 = 'Open bracket cutting error'
        self.string_4 = 'Closed bracket cutting error'
        self.string_5 = '{'
        self.DwvbpfyYn = '}'
        self.ExecSimlpeMacrossDeligate = None

        self.string_5 = openBracket
        self.DwvbpfyYn = closeBracket
        self.ExecSimlpeMacrossDeligate = execSimlpeMacrossDeligate
        # self.regex_0 = re.compile(re.escape(self.string_5) + "[\\w\\W]*?" + re.escape(self.DwvbpfyYn))
        self.regex_0 = re.compile(re.escape(self.string_5) + "[\\w\\W]*?" + re.escape(self.DwvbpfyYn))

    def ExecComplexMacross(self, value :str):
        
        # if self.ExecSimlpeMacrossDeligate is None:
        #     return ''
        
        if not value:
            return ''

        s = value
        _input =''
        while s != _input:
            #_input =''
            s = self.regex_0.sub(self.method_0, s)
            _input = s
        return s


    
    def method_0(self, match :Match):

        str1 = ''
        str2 = ''
        try:
            # возможен баг. возможно надо в слайсинге добавить -1 на конце
            str2 = match.group(0)[len(self.string_5) : len(match.group(0)) - len(self.string_5)]
        except:
            return ''

        str3 = self.ExecComplexMacross(str(str2))
        if not (str3 == str2):
            return self.string_5 + str3
        
        str4 = ''
        try:
            #str4 = str2[0 : len(str2) - len(self.DwvbpfyYn)]
            str4 = str2[0 : len(str2)]
        except:
            return ''
        
        str1 = nickgen.login_macros_handler.LoginMacrossHandler.ExecSimplMacross(str(str4))
        return str1

    def SetExecSimlpeMacrossDeligate(self, delegate):
        self.ExecSimlpeMacrossDeligate = delegate



from nickgen.syllable_eng import SyllableEng
from nickgen.syllable_jap import SyllableJap
import os
from importlib.resources import files


class SyllableLoader:

    @staticmethod
    def LoadEng():
        # currentDir = os.path.dirname(os.path.abspath(__file__))
        # eng_path = os.path.join(currentDir, 'eng.json')
        
        eng_path = files('nickgen').joinpath('data').joinpath('eng.json')

        try:
            if not os.path.exists(eng_path):
                raise Exception(f'File not exists: {eng_path}')
            return SyllableEng.Load(eng_path)
        except:
            raise
            import sys;
            print("Unexpected error:", sys.exc_info()[0])
            raise Exception('syllableEng_0 = SyllableEng.Load')

    
    @staticmethod
    def LoadLat():
        # currentDir = os.path.dirname(os.path.abspath(__file__))
        # lat_path = os.path.join(currentDir, 'lat.json')
        
        lat_path = files('nickgen').joinpath('data').joinpath('lat.json')

        try:
            return SyllableEng.Load(lat_path)
        except:
            raise Exception('syllableEng_1 = SyllableEng.Load')


    @staticmethod
    def LoadJap():
        # currentDir = os.path.dirname(os.path.abspath(__file__))
        # jap_path = os.path.join(currentDir, 'jap.json')
        
        jap_path = files('nickgen').joinpath('data').joinpath('jap.json')
        
        try:
            return SyllableJap.Load(jap_path)
        except:
            #raise
            raise Exception('syllableJap_0 = SyllableJap.Load')


from nickgen.ForSyllableGroups import ForSyllableGroups
from nickgen.GroupChain import GroupChain
import random


class SyllableGroups:

    def __init__(self):
        self.fsg: ForSyllableGroups = ForSyllableGroups()
        #self.random_0 = random

    def AddSyllableInGroup(self, group: str, syllable: str):
        if group in self.fsg.syllableGroup:
            self.fsg.syllableGroup[group] = syllable
        else:
            self.fsg.syllableGroup[group] = []

    def AddForwardGroup(self, groupFirst: str, groupSecond: str):
        if groupFirst in self.fsg.forwardGroup:
            self.fsg.forwardGroup[groupFirst].AddForwardGroup(groupSecond)
        else:
            groupChain: GroupChain = GroupChain(groupFirst, groupFirst)
            groupChain.AddForwardGroup(groupSecond)

    def GetMatrix(self):
        # заменить на numpy.array ???
        stringList = []
        numArray = [[], []]

        for key in self.fsg.forwardGroup.keys():
            stringList.append(key)

        for i, item in enumerate(stringList):
            numArray[i] = []

            for i2, item2 in enumerate(stringList):
                numArray[i][i2] = self.fsg.forwardGroup[item].GetCount(item2)
                
        return [stringList, numArray]

    def GetSyllableByGroup(self, group: str):
        if group in self.fsg.syllableGroup:
            return self.fsg.syllableGroup[group]
        return [str]

    def GetGroups(self):
        return self.fsg.syllableGroup.keys()

    def GetLogins(self, length: int, count: int):

        stringList = []
        array = list(self.fsg.syllableGroup.keys())

        for _ in range(count):
            s = ''
            # !!!!!!!!!!!! Check rand(a, b-1)
            index2 = random.randint(0, len(self.fsg.syllableGroup) - 1)
            string_0 = array[index2]
            for _ in range(length):
                s += self.method_2(string_0)
                string_0 = self.method_1(string_0)
            stringList.append(s)

        return stringList

    def method_1(self, string_0: str):
        array1 = list(self.fsg.forwardGroup[string_0].ForwardGroups.values())
        array2 = list(self.fsg.forwardGroup[string_0].ForwardGroups.keys())
        maxValue = 0

        num1 = 0
        for i in range(len(array1) - 1):
            maxValue += array1[i]

        num2 = random.randint(0, maxValue - 1)
        num3 = 0
        for i in range(len(array1) - 1):
            num3 += array1[i]
            if (num1 <= num2 and num2 < num3):
                return array2[i]
            num1 = num3
        return ''

    def method_2(self, string_0: str):
        if string_0 not in self.fsg.syllableGroup:
            return ''
        stringList = self.fsg.syllableGroup[string_0]
        return stringList[random.randint(0, len(stringList)-1)]

    def GoogleTest(self, letters: str, countUse: str):
        # вроде бы не нужен
        pass



from typing import DefaultDict, Dict

from nickgen.GroupChain import GroupChain


class ForSyllableGroups:
    def __init__(self):
        # <str,List<str>>
        self.syllableGroup = {}
        # <str,GroupChain>
        self.forwardGroup = {}

        # Dictionary<string, List<string>> syllableGroup = new Dictionary<string, List<string>>();
        # Dictionary<string, GroupChain> forwardGroup = new Dictionary<string, GroupChain>();

    @classmethod
    def from_json(cls, data):
        # import json
        # fp = open('test.json', 'w')
        # json.dump(data, fp, indent=4)
        # fp.close()
        try:
            result = ForSyllableGroups()
            # result = cls.__new__(cls)

            # result.syllableGroup = Dict()
            # result.forwardGroup = Dict()

            syllableGroup_item = data["syllableGroup"]
            for key in syllableGroup_item.keys():
                value = syllableGroup_item[key]
                result.syllableGroup[key] = value

            forwardGroup_item = data["forwardGroup"]
            for key in forwardGroup_item.keys():
                value = forwardGroup_item[key]
                # value1 = value["forwardGroups"]
                value1_obj_dict = value["forwardGroups"]
                value1_obj_name = value["name"]
                value1_obj_Name = value["Name"]
                value1_obj_Dict = value["ForwardGroups"]

                gChainObj = GroupChain(value1_obj_name, value1_obj_Name)
                gChainObj.forwardGroups = value1_obj_dict
                gChainObj.ForwardGroups = value1_obj_Dict

                result.forwardGroup[key] = gChainObj

            return result
        except:
            raise



from setuptools import setup, find_packages
setup(
    name='nickgen',
    version='0.0.1',
    packages=find_packages(where="src"),
    package_dir={"": "src"},
    # package_data={"nickgen.data": ["*.json"]}
    include_package_data=True
)



from typing import Dict


class SyllableChain:

    def __init__(self, name: str):
        self.dictionary_0: Dict[str,int] = Dict()
        self.string_0: str = name

        #properties
        self.Name: str
        self.ForwardSyllables: Dict[str,int]

    def AddForwardSyllable(self, syllable: str):
        if syllable in self.dictionary_0:
            self.dictionary_0[syllable] = self.dictionary_0[syllable] + 1
        else:
            self.dictionary_0[syllable] = 1

    def GetCount(self, syllable: str):
        if syllable in self.dictionary_0:
            return self.dictionary_0[syllable]
        return 0


from typing import Dict

class GroupChain:
    def __init__(self, name :str, Name: str):
        self.name = name
        self.Name = Name
        self.forwardGroups :Dict[str,int] = dict()
        self.ForwardGroups :Dict[str,int] = dict()

    def AddForwardGroup(self, group :str):
        if group in self.forwardGroups:
            self.forwardGroups[group] = self.forwardGroups[group] + 1
            self.ForwardGroups[group] = self.ForwardGroups[group] + 1
        else:
            self.forwardGroups[group] = 1
            self.ForwardGroups[group] = 1

    def GetCount(self, group :str):
        if group in self.forwardGroups:
            return self.forwardGroups[group]
        else:
            return 0



from nickgen.login_macros_handler import LoginMacrossHandler


def generate(template="[Eng|4][RndNum|1970|1990]") -> str:
    """[Eng|4][RndNum|1970|1990]\n
    [Jap|4][RndNum|1970|1990]\n
    [Lat|4][RndNum|1970|1990]\n

    На данный момент поддерживаются языки Eng - английский, Lat - латынь, Jap - японский.
    То есть написав [Eng|4], будет сгенерирован никнейм длиной в 4 английских слога, с вероятностью следования слогов такой же как в реальных словах. Поигравшись с формулой генерации, можно создать более сложные конструкции:
    [RndSym|[RndNum|0|4]|0123456789][Lat|3][RndSym|[RndNum|0|2]|-][Jap|1][RndText|2|D]
    где [RndSym|[RndNum|0|4]|0123456789] - в начале ника идет от 0 до трех цифр;
    [Lat|3] 3 слога на латыни;
    [RndSym|[RndNum|0|2]|-] возможно появление дефиса;
    [Jap|1] один японский слог;
    [RndText|2|D] потом случайные 2 буквы или цифры;

    В результате будут сгенерированы ники:
        053bomenca-iem\n
        7lialeme-nozr\n
        46atbemig-poex\n
        simpvido-se8f\n
        3afosuxhif6\n
        frigulimdeif\n
        misssefu-yucn\n
        5grasacin-maew\n
        trodalcelfu88\n
        6nasercia-risc

    Args:
        template (str, optional): _description_. Defaults to '[Eng|4][RndNum|1970|1990]'.

    Returns:
        str: _description_
    """
    return LoginMacrossHandler.GetLogins(template, 1)[0]


def generate_many(count=1, template="[Eng|4][RndNum|1970|1990]") -> tuple[str]:
    """Смотри описание метода generate()

    Args:
        count (int, optional): Amount logins. Defaults to 1.
        template (str, optional): Generation template. Defaults to '[Eng|4][RndNum|1970|1990]'.

    Returns:
        tuple[str]: _description_
    """
    return LoginMacrossHandler.GetLogins(template, count)
